//handles the acutal blurring of the images
//Tim Bahls

public class BlurMaster {

	
	private static short MY_WEIGHT = 1;
	private static boolean VARIABLE_BLUR = true;
	
	private static  short[][][] nextData;

	private static final int[][] blurDataOne = { { 1, 1 }, { 1, 0 }, { 1, -1 }, { 0, -1 },
			{ -1, -1 }, { -1, 0 }, { -1, 1 }, { 0, 1 } };

	private static final int[][] blurDataTwo = { { -2, 2 }, { -2, 1 }, { -1, 1 }, { -1, 2 },
			{ 0, 2 }, { 0, 1 }, { 2, 2 }, { 1, 2 }, { 1, 1 }, { 2, 1 },
			{ 2, 0 }, { 1, 0 }, { 2, -2 }, { 2, -1 }, { 1, -1 }, { 1, -2 },
			{ 0, -2 }, { 0, -1 }, { -2, -2 }, { -1, -2 }, { -1, -1 },
			{ -2, -1 }, { -2, 0 }, { -1, 0 } };
	
	
	public static void blur(short[][][] data, int numIterations, int radius,
			int blurThresh) {

		nextData = new short[data.length][data[0].length][3];

		int[][] blurData = null;

		if (radius == 1) {
			blurData = blurDataOne;

		} else if (radius == 2) {
			blurData = blurDataTwo;
		} else {
			System.err.println("No such radius " + radius);
			System.exit(1);
		}
		for (int i = 0; i < numIterations; i++) {

			int bblurThresh = blurThresh;
			if (VARIABLE_BLUR) {
				bblurThresh = (blurThresh * i) / numIterations + blurThresh / 2;
			}
			blurOnce(data, radius, bblurThresh * bblurThresh, blurData);
		}
	}

	public static void blurOnce(short[][][] data, int rad, int blurThresh,
			int[][] stencilData) {
		int len = stencilData.length;
		int halfLen = len / 2;

		int hrz = data.length;
		int vrt = data[0].length;

		for (int x = rad; x < hrz - rad; x++) {
			for (int y = rad; y < vrt - rad; y++) {
				boolean rev = false;
				int minDist = Integer.MAX_VALUE;
				int maxDist = 0;
				int minInt = 0;

				short[] sum = new short[3];
				short[] pix = data[x][y];
				short[] total = new short[3];
				for (int c = 0; c < 3; c++) {
					for (int i = -rad; i <= rad; i++) {
						for (int j = -rad; j <= rad; j++) {
							total[c] += data[x + i][y + j][c];
						}

					}
					total[c] -= data[x][y][c];
				}
				for (int i = 0; i < halfLen; i++) {
					for (int c = 0; c < 3; c++) {
						sum[c] += data[x + stencilData[i][0]][y
								+ stencilData[i][1]][c];
					}
				}
				for (int i = 0; i < halfLen; i++) {

					int dist = 0;
					for (int k = 0; k < 3; k++) {
						int d = pix[k] - sum[k] / halfLen;
						dist += d * d;

					}
					if (dist < minDist) {
						minDist = dist;
						minInt = i;
						rev = false;
					}
					if (dist > maxDist)
						maxDist = dist;

					for (int c = 0; c < 3; c++)
						sum[c] = (short) (total[c] - sum[c]);

					dist = 0;

					for (int k = 0; k < 3; k++) {
						int d = pix[k] - sum[k] / halfLen;
						dist += d * d;

					}

					if (dist < minDist) {
						minDist = dist;
						minInt = i;
						rev = true;

					}
					if (dist > maxDist)
						maxDist = dist;

					for (int c = 0; c < 3; c++)
						sum[c] = (short) (total[c] - sum[c]);

					for (int k = 0; k < 3; k++) {
						sum[k] -= data[x + stencilData[i][0]][y
								+ stencilData[i][1]][k];
						int on = (i + halfLen) % len;
						sum[k] += data[x + stencilData[on][0]][y
								+ stencilData[on][1]][k];

					}

				}

				if (maxDist < blurThresh) {
					for (int c = 0; c < 3; c++) {
						int theSum = total[c] + MY_WEIGHT * data[x][y][c];
						nextData[x][y][c] = (short) (theSum / (len + MY_WEIGHT));

					}
				} else {
					sum = new short[3];

					int offSet = minInt;

					if (rev) {
						offSet += halfLen;
					}
					for (int c = 0; c < 3; c++) {
						for (int i = 0; i < halfLen; i++) {

							int ii = (i + offSet) % len;
							sum[c] += data[x + stencilData[ii][0]][y
									+ stencilData[ii][1]][c];
						}
						sum[c] += MY_WEIGHT * data[x][y][c];
						nextData[x][y][c] = (short) (sum[c] / (halfLen + MY_WEIGHT));
					}

				}

			}
		}
		short[][][] tmp = data;
		data = nextData;
		nextData = tmp;

	}
	
}
